
#include  "EdWgfx.h"
#include <Arduino.h>
#include <SPI.h>


#define swap(x,y) { x = x + y; y = x - y; x = x - y; }


EdWgfx::EdWgfx( uint8_t csPin, uint8_t busyPin )
  :    csPin_(csPin)
  ,    busyPin_(busyPin)
{
  //----- Inicjalizacja interfejsu SPI -----//
  pinMode(csPin_, OUTPUT);
  SPI.begin();
  SPI.setClockDivider(SPI_CLOCK_DIV2);   // 8MHz
}


//------------------------------------------------------------------------//
// Wysyla/odbiera 4 bajty danych po SPI                                   //
// Zależnie od argumentu "cs" kasuje/ustawia strob /SS                    //
//  i jak trzeba czeka na INT/BUSY                                        //
//------------------------------------------------------------------------//
void EdWgfx::SendReciv4spi( uint8_t s1, uint8_t s2, uint8_t s3, uint8_t s4, uint8_t *r1, uint8_t *r2, uint8_t *r3, uint8_t *r4, uint8_t cs ) {
  if ( cs & 1 ) digitalWrite(csPin_, LOW);  // CS=L

  SPDR = s1; while ( !(SPSR & (1 << SPIF))); *r1 = SPDR;
  SPDR = s2; while ( !(SPSR & (1 << SPIF))); *r2 = SPDR;
  SPDR = s3; while ( !(SPSR & (1 << SPIF))); *r3 = SPDR;
  SPDR = s4; while ( !(SPSR & (1 << SPIF))); *r4 = SPDR;

  if ( cs & 2 ) {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }
}


//------------------------------------------------------------------------//
void EdWgfx::Send4spi(uint8_t s1, uint8_t s2, uint8_t s3, uint8_t s4, uint8_t cs) {

  if ( cs & 1 ) digitalWrite(csPin_, LOW);  // CS=L

  SPDR = s1; while ( !(SPSR & (1 << SPIF)));
  SPDR = s2; while ( !(SPSR & (1 << SPIF)));
  SPDR = s3; while ( !(SPSR & (1 << SPIF)));
  SPDR = s4; while ( !(SPSR & (1 << SPIF)));

  if ( cs & 2 ) {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }
}


void EdWgfx::begin() {
  uint8_t buf[GFX_TURBO_ID_SIZE];

  readId( buf );
  readId( buf );

  // todo: cos zle czyta, w analizatorze ok. test w "benchmark.ino" ok
  rozdzielczoscX = buf[16] | buf[17] << 8;
  rozdzielczoscY = buf[18] | buf[19] << 8;

  // todo: rozmiar na stale
  rozdzielczoscX = rozdzielczoscY = 128;
}


void EdWgfx::ofs( uint8_t ofsX, uint8_t ofsY ) {
  Send4spi( CMDd_OFS, ofsX, ofsY, 0, 3 );
}


void EdWgfx::setContinue(uint8_t c) {
  cont = c;
}


void EdWgfx::display() {
  //----- Zawsze konczymy strob CS, bo moze tryb "COUNT"
  digitalWrite(csPin_, HIGH); // CS=H

  //----- Czekamy na zakończenie operacji przez akcelerator, nie ma sensu wysyłanie danych do LCD gdy nie wszystkie są gotowe
  while ( digitalRead(busyPin_) == LOW) ; // Czekaj na nieaktywne INT/BUSY

  //----- cmd DISPLAY w standardowy sposów (CS=L, CS=H + wait BUSY
  Send4spi( CMDd_DISPLAY, 0, 0, 0, 3 );

  cont = false;
}


void EdWgfx::nop() {
  //----- Zawsze w standardowy sposób
  Send4spi( CMDd_NOP, 0, 0, 0, 3 ); // Zawsze CS=H i wait BUSY
  cont = false;
}


void EdWgfx::fillScreen( uint8_t kolor ) {
  if ( cont )
    Send4spi( CMDd_CLS + CMDd_CD, kolor, 0, 0, 1 ); // Bez CS=H + wait BUSY
  else
    Send4spi( CMDd_CLS, kolor, 0, 0, 3 );

  ColorBack = kolor;
}


void EdWgfx::setAddrWindow(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1) {
  byte cmd = CMDd_WINDOW;
  if ( cont ) cmd += CMDd_CD;

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = cmd; while ( !(SPSR & (1 << SPIF)));
  SPDR = x0; while ( !(SPSR & (1 << SPIF)));
  SPDR = y0; while ( !(SPSR & (1 << SPIF)));
  SPDR = x1; while ( !(SPSR & (1 << SPIF)));
  SPDR = y1; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4

  if ( ! cont ) {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }
}


void EdWgfx::drawSprite(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, uint8_t * dat, uint16_t len) {
  cont = false;  // Tu tryb "CONT" niedziała bo po CMD nie znana liczba danych

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = CMDd_SPRITE; while ( !(SPSR & (1 << SPIF)));
  SPDR = x0; while ( !(SPSR & (1 << SPIF)));
  SPDR = y0; while ( !(SPSR & (1 << SPIF)));
  SPDR = x1; while ( !(SPSR & (1 << SPIF)));
  SPDR = y1; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  for (uint16_t c = 0; c < len; c++) {
    SPDR = dat[c]; while ( !(SPSR & (1 << SPIF)));
  }
  digitalWrite(csPin_, HIGH); // CS=H
  while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
}


void EdWgfx::drawSprite_P(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, uint8_t * dat, uint16_t len) {
  cont = false;  // Tu tryb "CONT" niedziała bo po CMD nie znana liczba danych

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = CMDd_SPRITE; while ( !(SPSR & (1 << SPIF)));
  SPDR = x0; while ( !(SPSR & (1 << SPIF)));
  SPDR = y0; while ( !(SPSR & (1 << SPIF)));
  SPDR = x1; while ( !(SPSR & (1 << SPIF)));
  SPDR = y1; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  for (uint16_t c = 0; c < len; c++) {
    SPDR = pgm_read_byte( &(dat[c]) ); while ( !(SPSR & (1 << SPIF)));
  }
  digitalWrite(csPin_, HIGH); // CS=H
  while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
}


#if defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)
void EdWgfx::drawSprite_PF(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, uint8_t * dat, uint16_t len) {
  cont = false; // Tu tryb "CONT" niedziała bo po CMD nie znana liczba danych

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = CMDd_SPRITE; while ( !(SPSR & (1 << SPIF)));
  SPDR = x0; while ( !(SPSR & (1 << SPIF)));
  SPDR = y0; while ( !(SPSR & (1 << SPIF)));
  SPDR = x1; while ( !(SPSR & (1 << SPIF)));
  SPDR = y1; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  for (uint16_t c = 0; c < len; c++) {
    SPDR = pgm_read_byte_far( &(dat[c]) ); while ( !(SPSR & (1 << SPIF)));
  }
  digitalWrite(csPin_, HIGH); // CS=H
  while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
}
#endif


/*
  void EdWgfx::drawSprite_A(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1, __memx * dat, uint16_t len) {
  cont = false;  // Tu tryb "CONT" niedziała bo po CMD nie znana liczba danych

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = CMDd_SPRITE; while ( !(SPSR & (1 << SPIF)));
  SPDR = x0; while ( !(SPSR & (1 << SPIF)));
  SPDR = y0; while ( !(SPSR & (1 << SPIF)));
  SPDR = x1; while ( !(SPSR & (1 << SPIF)));
  SPDR = y1; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  for (uint16_t c = 0; c < len; c++) {
    SPDR = dat[c]; while ( !(SPSR & (1 << SPIF)));
  }
  digitalWrite(csPin_, HIGH); // CS=H
  while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }
*/


void EdWgfx::setCursor(uint8_t x, uint8_t y) {
  byte cmd = CMDd_CURSOR;
  if ( cont ) cmd += CMDd_CD;

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = cmd; while ( !(SPSR & (1 << SPIF)));
  SPDR = CursX = x; while ( !(SPSR & (1 << SPIF)));
  SPDR = CursY = y; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF)));
  if ( ! cont ) {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }
}


void EdWgfx::printString(char *txt) {
  uint8_t len = strlen(txt);


  if ( cont ) EdWgfx::nop(); // todo: na razie konczymy CS, bo cosnie dziala CMD po string
  // prawdopodobnie problemem jest dopelnianie do 4 bajtów i późniejsze traktowanie przez akcelerator danych jak komendy


  byte cmd = CMDd_STRING;
  if ( cont ) cmd += CMDd_CD;


  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = cmd; while ( !(SPSR & (1 << SPIF)));
  SPDR = CursX; while ( !(SPSR & (1 << SPIF)));
  SPDR = CursY; while ( !(SPSR & (1 << SPIF)));
  SPDR = txtSize; while ( !(SPSR & (1 << SPIF)));
  SPDR = txtAtrybuty; while ( !(SPSR & (1 << SPIF)));
  SPDR = ColorPen; while ( !(SPSR & (1 << SPIF)));
  SPDR = ColorBack; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  for (uint8_t x = 0; x < len; x++) {
    SPDR = txt[x]; while ( !(SPSR & (1 << SPIF)));
  }
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // Koniec tekstu


  byte r = ((len + 1) % 4);
  switch ( r ) {
    case 1:
      SPDR = 0; while ( !(SPSR & (1 << SPIF))); // Dopelniamy łacznie 3 bajtami
    case 2:
      SPDR = 0; while ( !(SPSR & (1 << SPIF))); // Dopelniamy łacznie 2 bajtami
    case 3:
      SPDR = 0; while ( !(SPSR & (1 << SPIF))); // Dopelniamy 1 bajtem
  }


  if ( ! cont )
  {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }
}


void EdWgfx::setTextAtrybut(uint8_t atrybuty) {
  txtAtrybuty = atrybuty;
}


void EdWgfx::setTextSize(uint8_t rozmiar) {
  txtSize = rozmiar;
}


void EdWgfx::setTextColor(uint8_t color) {
  ColorPen = color;
}


void EdWgfx::setTextBack(uint8_t color) {
  ColorBack = color;
}


void EdWgfx::drawPixel(uint8_t x, uint8_t y, uint8_t color) {
  if ( cont ) Send4spi( CMDd_PIXEL + CMDd_CD, x, y, color, 1 ); // CS=L, bez CS=H + Wait INT/BUSY
  else Send4spi( CMDd_PIXEL, x, y, color, 3 ); // CS=L, CS=H + Wait INT/BUSY
}


void EdWgfx::drawFastVLine(uint8_t x, uint8_t y, uint8_t h, uint8_t color) {
  byte cmd = CMDd_LINE_V;
  if ( cont ) cmd += CMDd_CD;

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = cmd; while ( !(SPSR & (1 << SPIF)));
  SPDR = x; while ( !(SPSR & (1 << SPIF)));
  SPDR = y; while ( !(SPSR & (1 << SPIF)));
  SPDR = h; while ( !(SPSR & (1 << SPIF)));
  SPDR = color; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  if ( ! cont ) {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }

  /*
    cont = false; // Tu tryb "CONT" niedziała bo po CMD nie znana liczba danych

    digitalWrite(csPin_, LOW);  // CS=L
    SPDR = CMDd_WINDOW; while ( !(SPSR & (1 << SPIF)));
    SPDR = x; while ( !(SPSR & (1 << SPIF)));
    SPDR = y; while ( !(SPSR & (1 << SPIF)));
    SPDR = x; while ( !(SPSR & (1 << SPIF)));
    SPDR = y + h; while ( !(SPSR & (1 << SPIF)));
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    for (uint8_t x = 0; x < h; x++) {
    SPDR = color; while ( !(SPSR & (1 << SPIF)));
    }
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  */
}


void EdWgfx::drawFastHLine(uint8_t x, uint8_t y, uint8_t w, uint8_t color) {

  byte cmd = CMDd_LINE_H;
  if ( cont ) cmd += CMDd_CD;

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = cmd; while ( !(SPSR & (1 << SPIF)));
  SPDR = x; while ( !(SPSR & (1 << SPIF)));
  SPDR = y; while ( !(SPSR & (1 << SPIF)));
  SPDR = w; while ( !(SPSR & (1 << SPIF)));
  SPDR = color; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  if ( ! cont ) {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }

  /*
    cont = false;   // Tu tryb "CONT" niedziała bo po CMD nie znana liczba danych

    digitalWrite(csPin_, LOW);  // CS=L
    SPDR = CMDd_WINDOW; while ( !(SPSR & (1 << SPIF)));
    SPDR = x; while ( !(SPSR & (1 << SPIF)));
    SPDR = y; while ( !(SPSR & (1 << SPIF)));
    SPDR = x + w; while ( !(SPSR & (1 << SPIF)));
    SPDR = y; while ( !(SPSR & (1 << SPIF)));
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    for (uint8_t x = 0; x < w; x++) {
    SPDR = color; while ( !(SPSR & (1 << SPIF)));
    }
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  */
}


void EdWgfx::drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint8_t color) {
  byte cmd = CMDd_LINE;
  if ( cont ) cmd += CMDd_CD;

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = cmd; while ( !(SPSR & (1 << SPIF)));
  SPDR = x0; while ( !(SPSR & (1 << SPIF)));
  SPDR = y0; while ( !(SPSR & (1 << SPIF)));
  SPDR = x1; while ( !(SPSR & (1 << SPIF)));
  SPDR = y1; while ( !(SPSR & (1 << SPIF)));
  SPDR = color; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  if ( ! cont ) {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }


  /*
    cont = false;   // Tu tryb "CONT" niedziała bo po CMD nie znana liczba danych

    if (y0 == y1) {
    if (x1 > x0) {
      drawFastHLine(x0, y0, x1 - x0 + 1, color);
    } else if (x1 < x0) {
      drawFastHLine(x1, y0, x0 - x1 + 1, color);
    } else {
      drawPixel(x0, y0, color);
    }
    return;
    } else if (x0 == x1) {
    if (y1 > y0) {
      drawFastVLine(x0, y0, y1 - y0 + 1, color);
    } else {
      drawFastVLine(x0, y1, y0 - y1 + 1, color);
    }
    return;
    }

    bool steep = abs(y1 - y0) > abs(x1 - x0);
    if (steep) {
    swap(x0, y0);
    swap(x1, y1);
    }
    if (x0 > x1) {
    swap(x0, x1);
    swap(y0, y1);
    }

    int16_t dx, dy;
    dx = x1 - x0;
    dy = abs(y1 - y0);

    int16_t err = dx / 2;
    int16_t ystep;

    if (y0 < y1) {
    ystep = 1;
    } else {
    ystep = -1;
    }


    int16_t xbegin = x0;
    if (steep) {
    for (; x0 <= x1; x0++) {
      err -= dy;
      if (err < 0) {
        int16_t len = x0 - xbegin;
        if (len) {
          drawFastVLine(y0, xbegin, len + 1, color);
        } else {
          drawPixel(y0, x0, color);
        }
        xbegin = x0 + 1;
        y0 += ystep;
        err += dx;
      }
    }
    if (x0 > xbegin + 1) {
      drawFastVLine(y0, xbegin, x0 - xbegin, color);
    }

    } else {
    for (; x0 <= x1; x0++) {
      err -= dy;
      if (err < 0) {
        int16_t len = x0 - xbegin;
        if (len) {
          drawFastHLine(xbegin, y0, len + 1, color);
        } else {
          drawPixel(x0, y0, color);
        }
        xbegin = x0 + 1;
        y0 += ystep;
        err += dx;
      }
    }
    if (x0 > xbegin + 1) {
      drawFastHLine(xbegin, y0, x0 - xbegin, color);
    }
    }
  */
}


void EdWgfx::drawRect(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t color) {
  EdWgfx::drawFastHLine(x, y, w, color);
  EdWgfx::drawFastHLine(x, y + h, w, color);

  EdWgfx::drawFastVLine(x, y, h, color);
  EdWgfx::drawFastVLine(x + w, y, h, color);
}


void EdWgfx::fillRect(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t color) {
  byte cmd = CMDd_FILL;
  if ( cont ) cmd += CMDd_CD;

  digitalWrite(csPin_, LOW);  // CS=L
  SPDR = cmd; while ( !(SPSR & (1 << SPIF)));
  SPDR = x; while ( !(SPSR & (1 << SPIF)));
  SPDR = y; while ( !(SPSR & (1 << SPIF)));
  SPDR = w; while ( !(SPSR & (1 << SPIF)));
  SPDR = h; while ( !(SPSR & (1 << SPIF)));
  SPDR = color; while ( !(SPSR & (1 << SPIF)));
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
  SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4

  if ( ! cont ) {
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  }

  /*
    cont = false; // Tu tryb "CONT" niedziała bo po CMD nie znana liczba danych

    digitalWrite(csPin_, LOW);  // CS=L
    SPDR = CMDd_WINDOW; while ( !(SPSR & (1 << SPIF)));
    SPDR = x; while ( !(SPSR & (1 << SPIF)));
    SPDR = y; while ( !(SPSR & (1 << SPIF)));
    SPDR = x + w; while ( !(SPSR & (1 << SPIF)));
    SPDR = y + h; while ( !(SPSR & (1 << SPIF)));
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    SPDR = 0; while ( !(SPSR & (1 << SPIF))); // !!! PUSTY - dopelnienie do 4
    for (uint16_t x = 0; x < w * h; x++) {
    SPDR = color; while ( !(SPSR & (1 << SPIF)));
    }
    digitalWrite(csPin_, HIGH); // CS=H
    while ( digitalRead(busyPin_) == LOW) ; // Czekaj na INT/BUSY
  */
}


void EdWgfx::readId( uint8_t *buf ) {
  uint8_t pusty;

  // " + CMDd_CD" bo komendy hurtem czy klient chce czy nie
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &pusty, &pusty, &buf[0], &buf[1], 1 ); // CS = L
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[2], &buf[3], &buf[4], &buf[5], 0 ); // CS nie ruszamy
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[6], &buf[7], &buf[8], &buf[9], 0 );
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[10], &buf[11], &buf[12], &buf[13], 0 );
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[14], &buf[15], &buf[16], &buf[17], 0 );
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[18], &buf[19], &buf[20], &buf[21], 0 );
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[22], &buf[23], &buf[24], &buf[25], 0 );
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[26], &buf[27], &buf[28], &buf[29], 0 );
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[30], &buf[31], &buf[32], &buf[33], 0 );
  SendReciv4spi( CMDd_NOP + CMDd_CD, 0, 0, 0, &buf[34], &buf[35], &buf[36], &buf[37], 2 ); // CS=H + Wait INT/BUSY
}


uint16_t EdWgfx::readAdrDMA() {
  uint8_t buf[GFX_TURBO_ID_SIZE];

  readId( buf );

  return buf[0] << 8 | buf[1];
}


uint8_t EdWgfx::readIN() {
  uint8_t buf[GFX_TURBO_ID_SIZE];

  readId( buf );

  return buf[37] >> 4;
}


uint8_t EdWgfx::readOUT() {
  uint8_t buf[GFX_TURBO_ID_SIZE];

  readId( buf );

  return buf[37] & 15;
}


uint8_t EdWgfx::readPWM() {
  uint8_t buf[GFX_TURBO_ID_SIZE];

  readId( buf );

  return buf[36];
}


void EdWgfx::setOUT(uint8_t out ) {
  Send4spi( CMDd_OUT, out, 0, 0, 3 );
}


void EdWgfx::setPWM(uint8_t pwm) {
  Send4spi( CMDd_PWM, pwm, 0, 0, 3 );
}


uint8_t EdWgfx::width() {
  return rozdzielczoscX;
}


uint8_t EdWgfx::height() {
  return rozdzielczoscY;
}


void EdWgfx::setRotation(uint8_t r) {
  //todo:
}
